from smart_module.module_system.module_factions import *

from smart_module.module_system.process_common import *

faction_name_pos = 0
faction_flags_pos = 2
faction_coherence_pos = 3
faction_relations_pos = 4
faction_ranks_pos = 5

def compile_relations():
  relations = []
  for i in range(len(factions)):
    r = [0.0 for j in range(len(factions))]
    relations.append(r)
  for i_faction in range(len(factions)):
    relations[i_faction][i_faction] = factions[i_faction][faction_coherence_pos]
    rels = factions[i_faction][faction_relations_pos]
    for rel in rels:
      rel_name = rel[0]
      other_pos = -1
      for j_f in range(len(factions)):
        if factions[j_f][faction_name_pos] == rel_name:
          other_pos = j_f
      if other_pos == -1:
        print( "ERROR faction not found: "+ rel_name)
      else:
        relations[other_pos][i_faction] = rel[1]
        relations[i_faction][other_pos] = rel[1]
  return relations

def save_factions(relations,export_dir):
  file = open(export_dir + "factions.txt","w",encoding="utf-8")
  file.write("factionsfile version 1\n")
  file.write("%d\n"%len(factions))
  for i_faction in range(len(factions)):
    faction = factions[i_faction]
    fac_color = 0xAAAAAA
    if len(faction) == 7:
      fac_color = faction[6]
    file.write("fac_%s %s %d %d \n"%(convert_to_identifier(faction[0]), replace_spaces(faction[1]), faction[2], fac_color))
    for reln in relations[i_faction]:
      file.write(" %f "%reln)
    file.write("\n")
    ranks = []
    if (len(faction) > (faction_ranks_pos)):
      ranks = faction[faction_ranks_pos]
    file.write("%d "%(len(ranks)))
    for rank in ranks:
      file.write(" %s "%(replace_spaces(rank)))
  file.close()

def two_to_pow(x):
  result = 1
  for i in range(x):
    result = result * 2
  return result

def save_python_header(src_dir):
  file = open(src_dir + "ID_factions.py","w",encoding="utf-8")
  for i_faction in range(len(factions)):
    file.write("fac_%s = %d\n"%(factions[i_faction][0],i_faction))
  file.write("\n\n")
  file.close()


def processFactions(context):
  print( "Exporting faction data...")
  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  relations = compile_relations()
  save_factions(relations,export_dir)
  



from smart_module.module_system.module_scripts import *


'''
    领主军队管理功能
    
    领主会对军队进行相应的管理
    当没有国家交战时，会全部出售俘虏
    当有少量国家交战时，会出售它国俘虏，并招募本国俘虏
    当有大量国家交战时，会招募全部俘虏
    也就是不交战时，钱最重要，交战时士兵最重要
    
    时间越长，领主的士兵等级会越高，战斗力也越强
    
    ，该功能只针对以下性格的领主：
    1.军事的
    2.好战的
    3.和善的
    4.正直的
    5.冷酷的（坏人也有比较厉害的）
    6.善良的
    
    【重点】：招募和升级都是需要钱的，没有钱，就不会招募和升级！！！！
    
    
    注：如果全部都对军事做管理，那玩家将很大机率打不过任何一个领主
'''

## 领主自动维护士兵时间24 * 1(1天)
lord_auto_collect_tax_interval = 24 * 1

## 每次招募士兵花费
lord_recruit_soldiers_cost = 1500

## 每次更新士兵花费
lord_update_soldiers_cost = 700

lordSoldiersManage = {
    "name":"LordSoldiersManage",
    "enable":True,
    "version": "v2.0.0",
    "desc":'''
        领主军队管理功能
        
        领主会对军队进行相应的管理
        当没有国家交战时，会全部出售俘虏
        当有少量国家交战时，会出售它国俘虏，并招募本国俘虏
        当有大量国家交战时，会招募全部俘虏
        也就是不交战时，钱最重要，交战时士兵最重要
        
        时间越长，领主的士兵等级会越高，战斗力也越强
        
        ，该功能只针对以下性格的领主：
        1.军事的
        2.好战的
        3.和善的
        4.正直的
        5.冷酷的（坏人也有比较厉害的）
        6.善良的
        
        【重点】：招募和升级都是需要钱的，没有钱，就不会招募和升级！！！！
        
        注：如果全部都对军事做管理，那玩家将很大机率打不过任何一个领主
    ''',
    "actions":[
        ("Append@triggers", [
            # ## 测试处理俘虏功能
            # (0,0,ti_once,[],[
            #     (try_for_range,":lord_no",lords_begin,lords_end),
            #         (troop_get_slot,":party",":lord_no",slot_troop_leaded_party),
            #         (ge,":party",0),
            #         (party_add_prisoners,":party","trp_swadian_knight",5),
            #         (party_add_prisoners,":party","trp_vaegir_knight",5),
            #         (party_add_prisoners,":party","trp_khergit_lancer",5),
            #         (party_add_prisoners,":party","trp_nord_champion",5),
            #         (party_add_prisoners,":party","trp_rhodok_sharpshooter",5),
            #         (party_add_prisoners,":party","trp_sarranid_mamluke",5),
            #     (try_end),
            # ]),
        ]),
        ("Append@simple_triggers", [
            ## 领主军队维护
            (lord_auto_collect_tax_interval,[
                (try_for_range,":lord_no",lords_begin,lords_end),
                    (this_or_next|troop_slot_eq,":lord_no",slot_lord_reputation_type,lrep_martial),
                    (this_or_next|troop_slot_eq,":lord_no",slot_lord_reputation_type,lrep_goodnatured),
                    (this_or_next|troop_slot_eq,":lord_no",slot_lord_reputation_type,lrep_upstanding),
                    (this_or_next|troop_slot_eq,":lord_no",slot_lord_reputation_type,lrep_benefactor),
                    (this_or_next|troop_slot_eq,":lord_no",slot_lord_reputation_type,lrep_selfrighteous),
                    (troop_slot_eq,":lord_no",slot_lord_reputation_type,lrep_quarrelsome),

                    (troop_get_slot,":party",":lord_no",slot_troop_leaded_party),
                    (gt,":party",0),

                    ## 军队士兵数量
                    (party_get_num_companions,":party_size",":party"),

                    ## 理想士兵数量
                    (call_script,"script_party_get_ideal_size",":party"),
                    (assign,":ideal_size",reg0),

                    ## 需要补充的士兵数量
                    (store_sub,":need_add",":ideal_size",":party_size"),

                    # (str_store_troop_name_link, s1, ":lord_no"),
                    # (assign,reg1,":need_add"),
                    # (display_message,"@{s1} 需 要 补 充 的 士 兵 数 量 为 : {reg1}"),
                    # (troop_get_slot,":wealth",":lord_no",slot_troop_wealth),
                    # (assign,reg1,":wealth"),
                    # (display_message,"@{s1} 财 富 为 {reg1}(处 理 俘 虏 前)"),

                    (party_get_num_prisoners,":prisoners_num",":party"),
                    (try_begin),
                        ## 如果有俘虏就处理
                        (gt,":prisoners_num",0),
                        ## 根据当前国情进行俘虏处理
                        (store_faction_of_troop, ":faction", ":lord_no"),
                        (call_script, "script_get_num_of_enemy_state", ":faction"),
                        (assign, ":enemy_state_num", reg0),
                        (try_begin),
                            ## 没有有任何国家交战
                            (eq, ":enemy_state_num", 0),
                            ## 不需要补充士兵
                            (le,":need_add",0),
                            ## 俘虏全部出售
                            (assign, ":handle", -1),
                            #(display_message,"@{s1} 出 售 全 部 俘 虏"),
                        (else_try),
                            ## 交战国家较少
                            (le, ":enemy_state_num", 2),
                            ## 不需要补充士兵
                            (le,":need_add",0),
                            ## 本国招募，它国出售
                            (assign, ":handle", 0),
                            #(display_message,"@{s1} 招 募 部 分 俘 虏"),
                        (else_try),
                            ## 交战国家较多
                            ## 全部招募
                            (assign, ":handle", 1),
                            #(display_message,"@{s1} 招 募 全 部 俘 虏"),
                        (try_end),
                        ## 【处理俘虏】
                        (call_script, "script_party_handle_prisoners", ":party", ":handle"),
                    (try_end),

                    ## 重新计算军队士兵数量
                    (party_get_num_companions,":party_size",":party"),
                    ## 军事强度
                    (call_script,"script_get_lord_strength",":lord_no"),
                    (assign,":strength",reg0),

                    ## 需要补充的士兵数量
                    (store_sub,":need_add",":ideal_size",":party_size"),

                    # (str_store_troop_name_link, s1, ":lord_no"),
                    # (assign,reg1,":need_add"),
                    # (display_message,"@{s1} 需 要 补 充 的 士 兵 数 量 为 : {reg1}"),
                    # (troop_get_slot,":wealth",":lord_no",slot_troop_wealth),
                    # (assign,reg1,":wealth"),
                    # (str_store_troop_name_link,s1,":lord_no"),
                    # (display_message,"@{s1} 财 富 为 {reg1} ( 招 募 士 兵 前 )"),
                    ## 士兵补充
                    (try_begin),
                        ## 士兵需要补充
                        (lt, ":party_size", ":ideal_size"),
                        ## 钱足够
                        (troop_slot_ge,":lord_no",slot_troop_wealth,lord_recruit_soldiers_cost),
                        ## 增加士兵
                        (call_script, "script_party_add_members", ":party", -1, ":strength",-1, 30, 50),
                        ## 减少钱财
                        (call_script,"script_update_lord_wealth",":lord_no",lord_recruit_soldiers_cost,-1),

                        # (str_store_troop_name_link, s1, ":lord_no"),
                        # (display_message,"@{s1} 补 充 士 兵"),
                        # (troop_get_slot,":wealth",":lord_no",slot_troop_wealth),
                        # (assign,reg1,":wealth"),
                        # (str_store_troop_name_link,s1,":lord_no"),
                        # (display_message,"@{s1} 财 富 为 {reg1}(招 募 士 兵 后)"),
                    (try_end),

                    ## 增加经验
                    (try_begin),
                        ## 钱足够
                        (troop_slot_ge, ":lord_no", slot_troop_wealth, lord_update_soldiers_cost),
                        ## 减少钱财
                        (call_script,"script_update_lord_wealth",":lord_no",lord_update_soldiers_cost,-1),
                        (call_script,"script_party_add_xp_and_upgrade",":party",":strength",70),

                        # (str_store_troop_name_link, s1, ":lord_no"),
                        # (display_message,"@{s1} 升 级 士 兵"),
                        # (troop_get_slot,":wealth",":lord_no",slot_troop_wealth),
                        # (assign,reg1,":wealth"),
                        # (str_store_troop_name_link,s1,":lord_no"),
                        # (display_message,"@{s1} 财 富 为 {reg1}(升 级 士 兵 后)"),
                    (try_end),
                    #(display_message,"@----------------------------------------------------------------------------------"),
                (try_end),
            ]),
        ]),
    ],
}


from setuptools import setup, find_packages

setup(
    name = "smart-module-test",
    version = "0.0.2",
    author = "ArcherTan",
    maintainer = "ArcherTan",
    author_email = "yunwei1237@163.com",
    url = "https://gitee.com/yunwei1237/smart-module.git",
    description = (
        '骑砍源码组织利器，一个py文件就是一个神奇的module，每一个module都是一个单独的功能，可以代表是一个庞大的功能模块，也可以是一些通用类代码的集合，且每一个module无论功能大小，代码多少，都可以在多个剧本之间达到极速移植！'
    ),
    packages = find_packages('smart_module'),
    package_dir = {'': 'smart_module'},
    include_package_data = True,
    # 排除所有 README.txt
    exclude_package_data = { '': ['README.txt'] },
)


anim_stand = 0
anim_stand_man = 1
anim_stand_player_first_person = 2
anim_jump = 3
anim_jump_loop = 4
anim_jump_end = 5
anim_jump_end_hard = 6
anim_stand_unarmed = 7
anim_stand_single = 8
anim_stand_greatsword = 9
anim_stand_staff = 10
anim_stand_crossbow = 11
anim_turn_right = 12
anim_turn_left = 13
anim_turn_right_single = 14
anim_turn_left_single = 15
anim_turn_right_staff = 16
anim_turn_left_staff = 17
anim_turn_right_greatsword = 18
anim_turn_left_greatsword = 19
anim_prepare_kick_0 = 20
anim_prepare_kick_1 = 21
anim_prepare_kick_2 = 22
anim_prepare_kick_3 = 23
anim_kick_right_leg = 24
anim_kick_left_leg = 25
anim_run_forward = 26
anim_run_forward_onehanded = 27
anim_run_forward_staff = 28
anim_run_forward_greatsword = 29
anim_run_forward_hips_right = 30
anim_run_forward_hips_left = 31
anim_run_forward_right = 32
anim_run_forward_right_onehanded = 33
anim_run_forward_right_staff = 34
anim_run_forward_right_greatsword = 35
anim_run_forward_right_hips_right = 36
anim_run_forward_right_hips_left = 37
anim_run_forward_left = 38
anim_run_forward_left_onehanded = 39
anim_run_forward_left_staff = 40
anim_run_forward_left_greatsword = 41
anim_run_forward_left_hips_right = 42
anim_run_forward_left_hips_left = 43
anim_run_backward = 44
anim_run_backward_onehanded = 45
anim_run_backward_staff = 46
anim_run_backward_greatsword = 47
anim_run_backward_hips_right = 48
anim_run_backward_hips_left = 49
anim_run_backward_right = 50
anim_run_backward_right_onehanded = 51
anim_run_backward_right_staff = 52
anim_run_backward_right_greatsword = 53
anim_run_backward_right_hips_right = 54
anim_run_backward_right_hips_left = 55
anim_run_backward_left = 56
anim_run_backward_left_onehanded = 57
anim_run_backward_left_staff = 58
anim_run_backward_left_greatsword = 59
anim_run_backward_left_hips_right = 60
anim_run_backward_left_hips_left = 61
anim_run_right = 62
anim_run_right_onehanded = 63
anim_run_right_twohanded = 64
anim_run_right_polearm = 65
anim_run_right_hips_right = 66
anim_run_right_hips_left = 67
anim_run_left = 68
anim_run_left_onehanded = 69
anim_run_left_twohanded = 70
anim_run_left_polearm = 71
anim_run_left_hips_right = 72
anim_run_left_hips_left = 73
anim_walk_forward = 74
anim_walk_forward_onehanded = 75
anim_walk_forward_staff = 76
anim_walk_forward_greatsword = 77
anim_walk_forward_hips_right = 78
anim_walk_forward_hips_left = 79
anim_walk_backward = 80
anim_walk_backward_onehanded = 81
anim_walk_backward_staff = 82
anim_walk_backward_greatsword = 83
anim_walk_backward_hips_right = 84
anim_walk_backward_hips_left = 85
anim_walk_right = 86
anim_walk_right_onehanded = 87
anim_walk_right_twohanded = 88
anim_walk_right_polearm = 89
anim_walk_right_hips_right = 90
anim_walk_right_hips_left = 91
anim_walk_left = 92
anim_walk_left_onehanded = 93
anim_walk_left_twohanded = 94
anim_walk_left_polearm = 95
anim_walk_left_hips_right = 96
anim_walk_left_hips_left = 97
anim_walk_forward_right = 98
anim_walk_forward_right_onehanded = 99
anim_walk_forward_right_twohanded = 100
anim_walk_forward_right_polearm = 101
anim_walk_forward_right_hips_right = 102
anim_walk_forward_right_hips_left = 103
anim_walk_forward_left = 104
anim_walk_forward_left_onehanded = 105
anim_walk_forward_left_twohanded = 106
anim_walk_forward_left_polearm = 107
anim_walk_forward_left_hips_right = 108
anim_walk_forward_left_hips_left = 109
anim_walk_backward_left = 110
anim_walk_backward_left_onehanded = 111
anim_walk_backward_left_twohanded = 112
anim_walk_backward_left_polearm = 113
anim_walk_backward_left_hips_right = 114
anim_walk_backward_left_hips_left = 115
anim_walk_backward_right = 116
anim_walk_backward_right_onehanded = 117
anim_walk_backward_right_twohanded = 118
anim_walk_backward_right_polearm = 119
anim_walk_backward_right_hips_right = 120
anim_walk_backward_right_hips_left = 121
anim_walk_forward_crouch = 122
anim_stand_to_crouch = 123
anim_crouch_to_stand = 124
anim_ride_0 = 125
anim_ride_1 = 126
anim_lancer_ride_1 = 127
anim_lancer_charge_parried = 128
anim_ride_2 = 129
anim_ride_3 = 130
anim_ride_4 = 131
anim_lancer_ride_4 = 132
anim_lancer_ride_4_no_shield = 133
anim_ride_rear = 134
anim_ride_spur = 135
anim_ride_jump = 136
anim_ride_jump_end = 137
anim_ride_turn_right = 138
anim_ride_turn_left = 139
anim_mount_horse = 140
anim_dismount_horse = 141
anim_lancer_ride_0 = 142
anim_equip_default = 143
anim_unequip_default = 144
anim_equip_sword = 145
anim_unequip_sword = 146
anim_equip_greatsword = 147
anim_unequip_greatsword = 148
anim_equip_axe_left_hip = 149
anim_unequip_axe_left_hip = 150
anim_equip_crossbow = 151
anim_unequip_crossbow = 152
anim_equip_spear = 153
anim_unequip_spear = 154
anim_equip_dagger_front_left = 155
anim_unequip_dagger_front_left = 156
anim_equip_dagger_front_right = 157
anim_unequip_dagger_front_right = 158
anim_equip_axe_back = 159
anim_unequip_axe_back = 160
anim_equip_revolver_right = 161
anim_unequip_revolver_right = 162
anim_equip_pistol_front_left = 163
anim_unequip_pistol_front_left = 164
anim_equip_katana = 165
anim_unequip_katana = 166
anim_equip_wakizashi = 167
anim_unequip_wakizashi = 168
anim_equip_shield = 169
anim_unequip_shield = 170
anim_equip_bow_back = 171
anim_unequip_bow_back = 172
anim_equip_bow_left_hip = 173
anim_unequip_bow_left_hip = 174
anim_cancel_attack_onehanded = 175
anim_cancel_attack_twohanded = 176
anim_cancel_attack_polearm = 177
anim_ready_bow = 178
anim_release_bow = 179
anim_ready_bow_mounted = 180
anim_release_bow_mounted = 181
anim_ready_crossbow = 182
anim_release_crossbow = 183
anim_reload_crossbow = 184
anim_reload_crossbow_horseback = 185
anim_ready_javelin = 186
anim_release_javelin = 187
anim_ready_throwing_knife = 188
anim_release_throwing_knife = 189
anim_ready_throwing_axe = 190
anim_release_throwing_axe = 191
anim_ready_stone = 192
anim_release_stone = 193
anim_ready_pistol = 194
anim_release_pistol = 195
anim_reload_pistol = 196
anim_ready_musket = 197
anim_release_musket = 198
anim_reload_musket = 199
anim_ready_swingright_fist = 200
anim_release_swingright_fist = 201
anim_release_swingright_fist_continue = 202
anim_blocked_swingright_fist = 203
anim_parried_swingright_fist = 204
anim_ready_swingleft_fist = 205
anim_release_swingleft_fist = 206
anim_release_swingleft_fist_continue = 207
anim_blocked_swingleft_fist = 208
anim_parried_swingleft_fist = 209
anim_ready_direct_fist = 210
anim_release_direct_fist = 211
anim_release_direct_fist_continue = 212
anim_blocked_direct_fist = 213
anim_parried_direct_fist = 214
anim_ready_uppercut_fist = 215
anim_release_uppercut_fist = 216
anim_release_uppercut_fist_continue = 217
anim_blocked_uppercut_fist = 218
anim_parried_uppercut_fist = 219
anim_ready_slashright_twohanded = 220
anim_release_slashright_twohanded = 221
anim_release_slashright_twohanded_continue = 222
anim_blocked_slashright_twohanded = 223
anim_parried_slashright_twohanded = 224
anim_ready_slashleft_twohanded = 225
anim_release_slashleft_twohanded = 226
anim_release_slashleft_twohanded_continue = 227
anim_blocked_slashleft_twohanded = 228
anim_parried_slashleft_twohanded = 229
anim_ready_thrust_twohanded = 230
anim_release_thrust_twohanded = 231
anim_release_thrust_twohanded_continue = 232
anim_blocked_thrust_twohanded = 233
anim_parried_thrust_twohanded = 234
anim_ready_overswing_twohanded = 235
anim_release_overswing_twohanded = 236
anim_release_overswing_twohanded_continue = 237
anim_blocked_overswing_twohanded = 238
anim_parried_overswing_twohanded = 239
anim_ready_thrust_onehanded = 240
anim_release_thrust_onehanded = 241
anim_release_thrust_onehanded_continue = 242
anim_blocked_thrust_onehanded = 243
anim_parried_thrust_onehanded = 244
anim_ready_thrust_onehanded_horseback = 245
anim_release_thrust_onehanded_horseback = 246
anim_release_thrust_onehanded_horseback_continue = 247
anim_blocked_thrust_onehanded_horseback = 248
anim_parried_thrust_onehanded_horseback = 249
anim_ready_thrust_onehanded_lance = 250
anim_release_thrust_onehanded_lance = 251
anim_release_thrust_onehanded_lance_continue = 252
anim_blocked_thrust_onehanded_lance = 253
anim_parried_thrust_onehanded_lance = 254
anim_ready_slashright_onehanded = 255
anim_release_slashright_onehanded = 256
anim_release_slashright_onehanded_continue = 257
anim_blocked_slashright_onehanded = 258
anim_parried_slashright_onehanded = 259
anim_ready_slashleft_onehanded = 260
anim_release_slashleft_onehanded = 261
anim_release_slashleft_onehanded_continue = 262
anim_blocked_slashleft_onehanded = 263
anim_parried_slashleft_onehanded = 264
anim_ready_overswing_onehanded = 265
anim_release_overswing_onehanded = 266
anim_release_overswing_onehanded_continue = 267
anim_blocked_overswing_onehanded = 268
anim_parried_overswing_onehanded = 269
anim_ready_slash_horseback_right = 270
anim_release_slash_horseback_right = 271
anim_release_slash_horseback_right_continue = 272
anim_blocked_slash_horseback_right = 273
anim_parried_slash_horseback_right = 274
anim_ready_slash_horseback_left = 275
anim_release_slash_horseback_left = 276
anim_release_slash_horseback_left_continue = 277
anim_blocked_slash_horseback_left = 278
anim_parried_slash_horseback_left = 279
anim_ready_slash_horseback_polearm_right = 280
anim_release_slash_horseback_polearm_right = 281
anim_release_slash_horseback_polearm_right_continue = 282
anim_blocked_slash_horseback_polearm_right = 283
anim_parried_slash_horseback_polearm_right = 284
anim_ready_slash_horseback_polearm_left = 285
anim_release_slash_horseback_polearm_left = 286
anim_release_slash_horseback_polearm_left_continue = 287
anim_blocked_slash_horseback_polearm_left = 288
anim_parried_slash_horseback_polearm_left = 289
anim_ready_overswing_staff = 290
anim_release_overswing_staff = 291
anim_release_overswing_staff_continue = 292
anim_blocked_overswing_staff = 293
anim_parried_overswing_staff = 294
anim_ready_thrust_staff = 295
anim_release_thrust_staff = 296
anim_release_thrust_staff_continue = 297
anim_blocked_thrust_staff = 298
anim_parried_thrust_staff = 299
anim_ready_slashleft_staff = 300
anim_release_slashleft_staff = 301
anim_release_slashleft_staff_continue = 302
anim_blocked_slashleft_staff = 303
anim_parried_slashleft_staff = 304
anim_ready_slashright_staff = 305
anim_release_slashright_staff = 306
anim_release_slashright_staff_continue = 307
anim_blocked_slashright_staff = 308
anim_parried_slashright_staff = 309
anim_defend_fist = 310
anim_defend_fist_keep = 311
anim_defend_fist_parry_1 = 312
anim_defend_fist_parry_2 = 313
anim_defend_fist_parry_3 = 314
anim_defend_shield_forward = 315
anim_defend_shield_up = 316
anim_defend_shield_right = 317
anim_defend_shield_left = 318
anim_defend_shield = 319
anim_defend_shield_keep = 320
anim_defend_shield_parry_1 = 321
anim_defend_shield_parry_2 = 322
anim_defend_shield_parry_3 = 323
anim_defend_forward_greatsword = 324
anim_defend_forward_greatsword_keep = 325
anim_defend_forward_greatsword_parry_1 = 326
anim_defend_forward_greatsword_parry_2 = 327
anim_defend_forward_greatsword_parry_3 = 328
anim_defend_up_twohanded = 329
anim_defend_up_twohanded_keep = 330
anim_defend_up_twohanded_parry_1 = 331
anim_defend_up_twohanded_parry_2 = 332
anim_defend_up_twohanded_parry_3 = 333
anim_defend_right_twohanded = 334
anim_defend_right_twohanded_keep = 335
anim_defend_right_twohanded_parry_1 = 336
anim_defend_right_twohanded_parry_2 = 337
anim_defend_right_twohanded_parry_3 = 338
anim_defend_left_twohanded = 339
anim_defend_left_twohanded_keep = 340
anim_defend_left_twohanded_parry_1 = 341
anim_defend_left_twohanded_parry_2 = 342
anim_defend_left_twohanded_parry_3 = 343
anim_defend_forward_onehanded = 344
anim_defend_forward_onehanded_keep = 345
anim_defend_forward_onehanded_parry_1 = 346
anim_defend_forward_onehanded_parry_2 = 347
anim_defend_forward_onehanded_parry_3 = 348
anim_defend_up_onehanded = 349
anim_defend_up_onehanded_keep = 350
anim_defend_up_onehanded_parry_1 = 351
anim_defend_up_onehanded_parry_2 = 352
anim_defend_up_onehanded_parry_3 = 353
anim_defend_right_onehanded = 354
anim_defend_right_onehanded_keep = 355
anim_defend_right_onehanded_parry_1 = 356
anim_defend_right_onehanded_parry_2 = 357
anim_defend_right_onehanded_parry_3 = 358
anim_defend_left_onehanded = 359
anim_defend_left_onehanded_keep = 360
anim_defend_left_onehanded_parry_1 = 361
anim_defend_left_onehanded_parry_2 = 362
anim_defend_left_onehanded_parry_3 = 363
anim_defend_forward_staff = 364
anim_defend_forward_staff_keep = 365
anim_defend_forward_staff_parry_1 = 366
anim_defend_forward_staff_parry_2 = 367
anim_defend_forward_staff_parry_3 = 368
anim_defend_up_staff = 369
anim_defend_up_staff_keep = 370
anim_defend_up_staff_parry_1 = 371
anim_defend_up_staff_parry_2 = 372
anim_defend_up_staff_parry_3 = 373
anim_defend_right_staff = 374
anim_defend_right_staff_keep = 375
anim_defend_right_staff_parry_1 = 376
anim_defend_right_staff_parry_2 = 377
anim_defend_right_staff_parry_3 = 378
anim_defend_left_staff = 379
anim_defend_left_staff_keep = 380
anim_defend_left_staff_parry_1 = 381
anim_defend_left_staff_parry_2 = 382
anim_defend_left_staff_parry_3 = 383
anim_strike_head_left = 384
anim_strike_head_right = 385
anim_strike_head_front = 386
anim_strike_head_back = 387
anim_strike_chest_left = 388
anim_strike_chest_right = 389
anim_strike_chest_front = 390
anim_strike_chest_back = 391
anim_strike_abdomen_left = 392
anim_strike_abdomen_right = 393
anim_strike_abdomen_front = 394
anim_strike_abdomen_back = 395
anim_strike_legs_left = 396
anim_strike_legs_right = 397
anim_strike_legs_front = 398
anim_strike_legs_back = 399
anim_strike2_head_left = 400
anim_strike2_head_right = 401
anim_strike2_head_front = 402
anim_strike2_head_back = 403
anim_strike2_chest_left = 404
anim_strike2_chest_right = 405
anim_strike2_chest_front = 406
anim_strike2_chest_back = 407
anim_strike2_abdomen_left = 408
anim_strike2_abdomen_right = 409
anim_strike2_abdomen_front = 410
anim_strike2_abdomen_back = 411
anim_strike2_legs_left = 412
anim_strike2_legs_right = 413
anim_strike2_legs_front = 414
anim_strike2_legs_back = 415
anim_strike3_head_left = 416
anim_strike3_head_right = 417
anim_strike3_head_front = 418
anim_strike3_head_back = 419
anim_strike3_chest_left = 420
anim_strike3_chest_right = 421
anim_strike3_chest_front = 422
anim_strike3_abdomen_left = 423
anim_strike3_abdomen_right = 424
anim_strike3_abdomen_front = 425
anim_strike3_abdomen_back = 426
anim_strike_head_front_left_reloc = 427
anim_fall_face_hold = 428
anim_fall_chest_front = 429
anim_fall_abdomen_hold_front = 430
anim_fall_head_front = 431
anim_fall_right_front = 432
anim_fall_body_back = 433
anim_fall_rider_right_forward = 434
anim_fall_rider_right = 435
anim_fall_rider_left = 436
anim_rider_fall_right = 437
anim_rider_fall_roll = 438
anim_strike_chest_front_stop = 439
anim_strike_fall_back_rise = 440
anim_strike_fall_back_rise_upper = 441
anim_cheer = 442
anim_cheer_stand = 443
anim_stand_townguard = 444
anim_stand_lady = 445
anim_stand_lord = 446
anim_dance = 447
anim_pose_1 = 448
anim_pose_2 = 449
anim_pose_3 = 450
anim_pose_4 = 451
anim_pose_5 = 452
anim_wedding_guest = 453
anim_wedding_guest_notr = 454
anim_wedding_guest_woman = 455
anim_wedding_dad_stairs = 456
anim_wedding_dad_walk = 457
anim_wedding_bride_stairs = 458
anim_wedding_bride_walk = 459
anim_wedding_groom_wait = 460
anim_wedding_groom_last = 461
anim_wedding_dad_last = 462
anim_wedding_bride_last = 463
anim_equip_bayonet = 464
anim_unequip_bayonet = 465
anim_crouch_unarmed = 466
anim_crouch_single = 467
anim_crouch_greatsword = 468
anim_crouch_staff = 469
anim_crouch_crossbow = 470
anim_crouch_ready_pistol = 471
anim_crouch_release_pistol = 472
anim_reload_musket_full = 473
anim_reload_musket_two_third = 474
anim_reload_musket_one_third = 475
anim_crouch_pike = 476
anim_crouch_pike_recover = 477
anim_ready_overswing_spear = 478
anim_release_overswing_spear = 479
anim_release_overswing_spear_continue = 480
anim_parried_overswing_spear = 481
anim_blocked_overswing_spear = 482
anim_reload_pistol_half = 483
anim_ready_overswing_musket = 484
anim_release_overswing_musket = 485
anim_release_overswing_musket_continue = 486
anim_parried_overswing_musket = 487
anim_blocked_overswing_musket = 488
anim_ready_thrust_musket = 489
anim_release_thrust_musket = 490
anim_release_thrust_musket_continue = 491
anim_parried_thrust_musket = 492
anim_blocked_thrust_musket = 493
anim_equip_pistol_melee = 494
anim_unequip_pistol_melee = 495
anim_unused_human_anim_44 = 496
anim_unused_human_anim_45 = 497
anim_unused_human_anim_46 = 498
anim_unused_human_anim_47 = 499
anim_unused_human_anim_48 = 500
anim_unused_human_anim_49 = 501
anim_unused_human_anim_50 = 502
anim_unused_human_anim_51 = 503
anim_unused_human_anim_52 = 504
anim_unused_human_anim_53 = 505
anim_unused_human_anim_54 = 506
anim_unused_human_anim_55 = 507
anim_unused_human_anim_56 = 508
anim_unused_human_anim_57 = 509
anim_unused_human_anim_58 = 510
anim_unused_human_anim_59 = 511
anim_unused_human_anim_60 = 512
anim_unused_human_anim_61 = 513
anim_unused_human_anim_62 = 514
anim_unused_human_anim_63 = 515
anim_unused_human_anim_64 = 516
anim_unused_human_anim_65 = 517
anim_unused_human_anim_66 = 518
anim_unused_human_anim_67 = 519
anim_unused_human_anim_68 = 520
anim_unused_human_anim_69 = 521
anim_unused_human_anim_70 = 522
anim_unused_human_anim_71 = 523
anim_unused_human_anim_72 = 524
anim_unused_human_anim_73 = 525
anim_unused_human_anim_74 = 526
anim_unused_human_anim_75 = 527
anim_unused_human_anim_76 = 528
anim_unused_human_anim_77 = 529
anim_unused_human_anim_78 = 530
anim_unused_human_anim_79 = 531
anim_unused_human_anim_80 = 532
anim_unused_human_anim_81 = 533
anim_unused_human_anim_82 = 534
anim_unused_human_anim_83 = 535
anim_unused_human_anim_84 = 536
anim_unused_human_anim_85 = 537
anim_unused_human_anim_86 = 538
anim_unused_human_anim_87 = 539
anim_unused_human_anim_88 = 540
anim_unused_human_anim_89 = 541
anim_unused_human_anim_90 = 542
anim_unused_human_anim_91 = 543
anim_unused_human_anim_92 = 544
anim_unused_human_anim_93 = 545
anim_unused_human_anim_94 = 546
anim_unused_human_anim_95 = 547
anim_unused_human_anim_96 = 548
anim_unused_human_anim_97 = 549
anim_unused_human_anim_98 = 550
anim_unused_human_anim_99 = 551
anim_unused_human_anim_100 = 552
anim_horse_stand = 553
anim_horse_pace_1 = 554
anim_horse_pace_2 = 555
anim_horse_pace_3 = 556
anim_horse_pace_4 = 557
anim_horse_walk_backward = 558
anim_horse_rear = 559
anim_horse_jump = 560
anim_horse_jump_end = 561
anim_horse_turn_right = 562
anim_horse_turn_left = 563
anim_horse_turn_right_head = 564
anim_horse_turn_left_head = 565
anim_horse_slow = 566
anim_horse_fall_in_place = 567
anim_horse_fall_right = 568
anim_horse_fall_roll = 569
anim_unused_horse_anim_1 = 570
anim_unused_horse_anim_2 = 571
anim_unused_horse_anim_3 = 572
anim_unused_horse_anim_4 = 573
anim_unused_horse_anim_5 = 574
anim_unused_horse_anim_6 = 575
anim_unused_horse_anim_7 = 576
anim_unused_horse_anim_8 = 577
anim_unused_horse_anim_9 = 578
anim_unused_horse_anim_10 = 579
anim_unused_horse_anim_11 = 580
anim_unused_horse_anim_12 = 581
anim_unused_horse_anim_13 = 582
anim_unused_horse_anim_14 = 583
anim_unused_horse_anim_15 = 584
anim_unused_horse_anim_16 = 585
anim_unused_horse_anim_17 = 586
anim_unused_horse_anim_18 = 587
anim_unused_horse_anim_19 = 588
anim_unused_horse_anim_20 = 589
anim_unused_horse_anim_21 = 590
anim_unused_horse_anim_22 = 591
anim_unused_horse_anim_23 = 592
anim_unused_horse_anim_24 = 593
anim_unused_horse_anim_25 = 594
anim_unused_horse_anim_26 = 595
anim_unused_horse_anim_27 = 596
anim_unused_horse_anim_28 = 597
anim_unused_horse_anim_29 = 598
anim_unused_horse_anim_30 = 599
anim_unused_horse_anim_31 = 600
anim_unused_horse_anim_32 = 601
anim_unused_horse_anim_33 = 602
anim_unused_horse_anim_34 = 603
anim_unused_horse_anim_35 = 604
anim_unused_horse_anim_36 = 605
anim_unused_horse_anim_37 = 606
anim_unused_horse_anim_38 = 607
anim_unused_horse_anim_39 = 608
anim_unused_horse_anim_40 = 609
anim_unused_horse_anim_41 = 610
anim_unused_horse_anim_42 = 611
anim_unused_horse_anim_43 = 612
anim_unused_horse_anim_44 = 613
anim_unused_horse_anim_45 = 614
anim_unused_horse_anim_46 = 615
anim_unused_horse_anim_47 = 616
anim_unused_horse_anim_48 = 617
anim_unused_horse_anim_49 = 618
anim_unused_horse_anim_50 = 619
anim_unused_horse_anim_51 = 620
anim_unused_horse_anim_52 = 621
anim_unused_horse_anim_53 = 622
anim_unused_horse_anim_54 = 623
anim_unused_horse_anim_55 = 624
anim_unused_horse_anim_56 = 625
anim_unused_horse_anim_57 = 626
anim_unused_horse_anim_58 = 627
anim_unused_horse_anim_59 = 628
anim_unused_horse_anim_60 = 629
anim_unused_horse_anim_61 = 630
anim_unused_horse_anim_62 = 631
anim_unused_horse_anim_63 = 632
anim_unused_horse_anim_64 = 633
anim_unused_horse_anim_65 = 634
anim_unused_horse_anim_66 = 635
anim_unused_horse_anim_67 = 636
anim_unused_horse_anim_68 = 637
anim_unused_horse_anim_69 = 638
anim_unused_horse_anim_70 = 639
anim_unused_horse_anim_71 = 640
anim_unused_horse_anim_72 = 641
anim_unused_horse_anim_73 = 642
anim_unused_horse_anim_74 = 643
anim_unused_horse_anim_75 = 644
anim_unused_horse_anim_76 = 645
anim_unused_horse_anim_77 = 646
anim_unused_horse_anim_78 = 647
anim_unused_horse_anim_79 = 648
anim_unused_horse_anim_80 = 649
anim_unused_horse_anim_81 = 650
anim_unused_horse_anim_82 = 651
anim_unused_horse_anim_83 = 652
anim_unused_horse_anim_84 = 653
anim_unused_horse_anim_85 = 654
anim_unused_horse_anim_86 = 655
anim_unused_horse_anim_87 = 656
anim_unused_horse_anim_88 = 657
anim_unused_horse_anim_89 = 658
anim_unused_horse_anim_90 = 659
anim_unused_horse_anim_91 = 660
anim_unused_horse_anim_92 = 661
anim_unused_horse_anim_93 = 662
anim_unused_horse_anim_94 = 663
anim_unused_horse_anim_95 = 664
anim_unused_horse_anim_96 = 665
anim_unused_horse_anim_97 = 666
anim_unused_horse_anim_98 = 667
anim_unused_horse_anim_99 = 668
anim_unused_horse_anim_100 = 669





###################################################
# header_triggers.py
# This file contains declarations for triggers
# DO NOT EDIT THIS FILE!
###################################################

#trigger interval flags

#ti_on_game_start         = -2.0
ti_simulate_battle       = -5.0 #can only be used in module_simple_triggers
ti_on_party_encounter    = -6.0 #can only be used in module_simple_triggers
ti_question_answered     = -8
ti_server_player_joined  = -15.0 #can only be used in module_mission_templates triggers
# Used only by the server in multiplayer mode
ti_on_multiplayer_mission_end = -16.0

# Trigger Param 1: player_no
ti_before_mission_start  = -19.0 #can only be used in module_mission_templates triggers
ti_after_mission_start   = -20.0 #can only be used in module_mission_templates triggers
ti_tab_pressed           = -21.0 #can only be used in module_mission_templates triggers
ti_inventory_key_pressed = -22.0 #can only be used in module_mission_templates triggers
ti_escape_pressed        = -23.0 #can only be used in module_mission_templates triggers
ti_battle_window_opened  = -24.0 #can only be used in module_mission_templates triggers
ti_on_agent_spawn        = -25.0 #can only be used in module_mission_templates triggers
ti_on_agent_killed_or_wounded = -26.0 #can only be used in module_mission_templates triggers
ti_on_agent_knocked_down = -27.0 #can only be used in module_mission_templates triggers
ti_on_agent_hit          = -28.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: damage inflicted agent_id
# Trigger Param 2: damage dealer agent_id
# Trigger Param 3: inflicted damage
# Register 0: damage dealer item_id
# Position Register 0: position of the blow
#                      rotation gives the direction of the blow
# Trigger result: if returned result is greater than or equal to zero, inflicted damage is set to the value specified by the module.

ti_on_player_exit        = -29.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: player_id

ti_on_leave_area         = -30.0 #can only be used in module_mission_templates triggers
ti_on_scene_prop_init    = -40.0 #can only be used in module_scene_props triggers
ti_on_init_scene_prop    = ti_on_scene_prop_init
# Trigger Param 1: prop instance number

ti_on_scene_prop_hit     = -42.0 #can only be used in module_scene_props triggers
# Trigger Param 1: prop instance number
# Trigger Param 2: hit damage
# Position Register 1: Hit Position
# Position Register 2: x holds attacker agent id

ti_on_scene_prop_destroy = -43.0 #can only be used in module_scene_props triggers
# Trigger Param 1: prop instance number

ti_on_scene_prop_use     = -44.0 #can only be used in module_scene_props triggers
# Trigger Param 1: user agent id
# Trigger Param 2: prop instance number

ti_on_scene_prop_is_animating = -45.0 #can only be used in module_scene_props triggers
# Trigger Param 1: prop instance number
# Trigger Param 2: remaining animation time

ti_on_scene_prop_animation_finished = -46.0 #can only be used in module_scene_props triggers
# Trigger Param 1: prop instance number

ti_on_scene_prop_start_use = -47.0 #can only be used in module_scene_props triggers
# Trigger Param 1: user agent id
# Trigger Param 2: prop instance number

ti_on_scene_prop_cancel_use = -48.0 #can only be used in module_scene_props triggers
# Trigger Param 1: user agent id
# Trigger Param 2: prop instance number

ti_on_init_item          = -50.0 #can only be used in module_items triggers
ti_on_weapon_attack      = -51.0 #can only be used in module_items triggers
# Trigger Param 1: attacker agent id
# Position Register 1: Weapon Item Position
ti_on_missile_hit        = -52.0 #can only be used in module_items triggers
# Position Register 1: Missile Position
# Trigger Param 1: shooter agent id
ti_on_item_picked_up     = -53.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: agent id
# Trigger Param 2: item id
# Trigger Param 3: scene prop id (will be deleted after this trigger)
ti_on_item_dropped       = -54.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: agent id
# Trigger Param 2: item id
# Trigger Param 3: scene prop id
ti_on_agent_mount        = -55.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: agent id
# Trigger Param 2: horse agent id
ti_on_agent_dismount     = -56.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: agent id
# Trigger Param 2: horse agent id
ti_on_item_wielded       = -57.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: agent id
# Trigger Param 2: item id
ti_on_item_unwielded     = -58.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: agent id
# Trigger Param 2: item id
ti_on_presentation_load  = -60.0 #can only be used in module_presentations triggers
ti_on_presentation_run   = -61.0 #can only be used in module_presentations triggers
# Trigger Param 1: current time in miliseconds
ti_on_presentation_event_state_change = -62.0 #can only be used in module_presentations triggers
# Trigger Param 1: id of the object that has the event
# Trigger Param 2: value (when available)
ti_on_presentation_mouse_enter_leave  = -63.0 #can only be used in module_presentations triggers
# Trigger Param 1: id of the object that mouse enters/leaves
# Trigger Param 2: 0 if mouse enters, 1 if mouse leaves
ti_on_presentation_mouse_press        = -64.0 #can only be used in module_presentations triggers
# Trigger Param 1: id of the object that mouse is pressed on
# Trigger Param 2: 0: left mouse button, 1 right mouse button, 2 middle mouse button
ti_on_init_map_icon                   = -70.0 #can only be used in module_map_icons triggers
# Trigger Param 1: id of the owner party
ti_on_order_issued					  = -71.0 #can only be used in module_mission_templates triggers
# Trigger Param 1: order no
# Trigger Param 2: agent id


ti_once        = 100000000.0

# Keys that can be checked by key_is_down and key_clicked

key_1 = 0x02
key_2 = 0x03
key_3 = 0x04
key_4 = 0x05
key_5 = 0x06
key_6 = 0x07
key_7 = 0x08
key_8 = 0x09
key_9 = 0x0a
key_0 = 0x0b
key_a = 0x1e
key_b = 0x30
key_c = 0x2e
key_d = 0x20
key_e = 0x12
key_f = 0x21
key_g = 0x22
key_h = 0x23
key_i = 0x17
key_j = 0x24
key_k = 0x25
key_l = 0x26
key_m = 0x32
key_n = 0x31
key_o = 0x18
key_p = 0x19
key_q = 0x10
key_r = 0x13
key_s = 0x1f
key_t = 0x14
key_u = 0x16
key_v = 0x2f
key_w = 0x11
key_x = 0x2d
key_y = 0x15
key_z = 0x2c
key_numpad_0 = 0x52
key_numpad_1 = 0x4f
key_numpad_2 = 0x50
key_numpad_3 = 0x51
key_numpad_4 = 0x4b
key_numpad_5 = 0x4c
key_numpad_6 = 0x4d
key_numpad_7 = 0x47
key_numpad_8 = 0x48
key_numpad_9 = 0x49
key_num_lock        = 0x45
key_numpad_slash    = 0xb5
key_numpad_multiply = 0x37
key_numpad_minus    = 0x4a
key_numpad_plus     = 0x4e
key_numpad_enter    = 0x9c
key_numpad_period   = 0x53
key_insert    = 0xd2
key_delete    = 0xd3
key_home      = 0xc7
key_end       = 0xcf
key_page_up   = 0xc9
key_page_down = 0xd1
key_up      = 0xc8
key_down    = 0xd0
key_left    = 0xcb
key_right   = 0xcd
key_f1  = 0x3b
key_f2  = 0x3c
key_f3  = 0x3d
key_f4  = 0x3e
key_f5  = 0x3f
key_f6  = 0x40
key_f7  = 0x41
key_f8  = 0x42
key_f9  = 0x43
key_f10 = 0x44
key_f11 = 0x57
key_f12 = 0x58
key_space         = 0x39
key_escape        = 0x01
key_enter         = 0x1c
key_tab           = 0x0f
key_back_space    = 0x0e
key_open_braces   = 0x1a
key_close_braces  = 0x1b
key_comma         = 0x33
key_period        = 0x34
key_slash         = 0x35
key_back_slash    = 0x2b
key_equals        = 0x0d
key_minus         = 0x0c
key_semicolon     = 0x27
key_apostrophe    = 0x28
key_tilde         = 0x29
key_caps_lock     = 0x3a
key_left_shift     = 0x2a
key_right_shift    = 0x36
key_left_control   = 0x1d
key_right_control  = 0x9d
key_left_alt       = 0x38
key_right_alt      = 0xb8
key_left_mouse_button   = 0xe0
key_right_mouse_button  = 0xe1
key_middle_mouse_button = 0xe2
key_mouse_button_4      = 0xe3
key_mouse_button_5      = 0xe4
key_mouse_button_6      = 0xe5
key_mouse_button_7      = 0xe6
key_mouse_button_8      = 0xe7
key_mouse_scroll_up     = 0xee
key_mouse_scroll_down   = 0xef
key_xbox_a              = 0xf0
key_xbox_b              = 0xf1
key_xbox_x              = 0xf2
key_xbox_y              = 0xf3
key_xbox_dpad_up        = 0xf4
key_xbox_dpad_down      = 0xf5
key_xbox_dpad_right     = 0xf6
key_xbox_dpad_left      = 0xf7
key_xbox_start          = 0xf8
key_xbox_back           = 0xf9
key_xbox_rbumber        = 0xfa
key_xbox_lbumber        = 0xfb
key_xbox_ltrigger       = 0xfc
key_xbox_rtrigger       = 0xfd
key_xbox_rstick         = 0xfe      
key_xbox_lstick         = 0xff


# Game keys that can be checked by game_key_is_down and game_key_clicked

gk_move_forward = 0
gk_move_backward = 1
gk_move_left = 2
gk_move_right = 3
gk_action = 4
gk_jump = 5
gk_attack = 6
gk_defend = 7
gk_kick = 8
#gk_parry_then_attack = 8
gk_toggle_weapon_mode = 9
gk_equip_weapon_1 = 10
gk_equip_weapon_2 = 11
gk_equip_weapon_3 = 12
gk_equip_weapon_4 = 13
gk_equip_primary_weapon = 14
gk_equip_secondary_weapon = 15
gk_drop_weapon = 16
gk_sheath_weapon = 17
gk_leave = 18
gk_zoom = 19
gk_view_char = 20
gk_cam_toggle = 21
gk_view_orders = 22
gk_order_1 = 23
gk_order_2 = 24
gk_order_3 = 25
gk_order_4 = 26
gk_order_5 = 27
gk_order_6 = 28
##gk_order_halt = 22
##gk_order_follow = 23
##gk_order_charge = 24
##gk_order_dismount = 25
##gk_order_hold_fire_toggle = 26
##gk_order_advance = 27
##gk_order_fall_back = 28
##gk_order_stand_closer = 29
##gk_order_spread_out = 30
##gk_order_blunt_weapons_toggle = 31
gk_everyone_hear = 29
gk_infantry_hear = 30
gk_archers_hear = 31
gk_cavalry_hear = 32
gk_group0_hear = gk_infantry_hear
gk_group1_hear = gk_archers_hear
gk_group2_hear = gk_cavalry_hear
gk_group3_hear = 33
gk_group4_hear = 34
gk_group5_hear = 35
gk_group6_hear = 36
gk_group7_hear = 37
gk_group8_hear = 38
gk_reverse_order_group = 39
gk_everyone_around_hear = 40
gk_mp_message_all = 41
gk_mp_message_team = 42
gk_character_window = 43
gk_inventory_window = 44
gk_party_window = 45
gk_quests_window = 46
gk_game_log_window = 47
gk_quick_save = 48
gk_crouch = 49
gk_order_7 = 50
gk_order_8 = 51

#trigger positions
#------------------------------------------------------
trigger_check_pos = 0 
trigger_delay_pos = 1
trigger_rearm_pos = 2
trigger_conditions_pos = 3
trigger_consequences_pos = 4

def ti_val(iv):
  return iv




from smart_module.module_system.module_quests import *


def save_quests(export_dir):
  ofile = open(export_dir + "quests.txt","w",encoding="utf-8")
  ofile.write("questsfile version 1\n")
  ofile.write("%d\n"%(len(quests)))
  for i_quest in range(len(quests)):
    quest = quests[i_quest]
    ofile.write("qst_%s %s %d "%(quest[0],(quest[1].replace(" ","_")),quest[2]))
    ofile.write("%s "%(quest[3].replace(" ","_")))
    ofile.write("\n")
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_quests.py","w",encoding="utf-8")
  for i_quest in range(len(quests)):
    ofile.write("qst_%s = %d\n"%(quests[i_quest][0],i_quest))
  for i_quest in range(len(quests)):
    ofile.write("qsttag_%s = %d\n"%(quests[i_quest][0],opmask_quest_index|i_quest))
  ofile.write("\n\n")
  ofile.close()


def processQuests(context):
  print( "Exporting quest data...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_quests(export_dir)
  save_python_header(src_dir)
  



from smart_module.module_system.module_info import *
from smart_module.module_system.module_skins import *
from smart_module.module_system.process_common import *

# WARNING: The following should be the same as the number in face_generator.h

num_voice_types = 2
#####################


def replace_spaces(s0):
  return s0.replace(" ","_")


def write_face_tex(ofile,tex_set):
  ofile.write(" %d "%len(tex_set)) 
  for tex in tex_set:
    color = tex[1]
    hair_mats = tex[2]
    hair_colors = []
    if len(tex) > 3:
      hair_colors = tex[3]
    ofile.write(" %s %d %d %d "%(tex[0],color, len(hair_mats), len(hair_colors)))
    for hair_mat in hair_mats:
      ofile.write(" %s "%(replace_spaces(hair_mat)))
    for hair_color in hair_colors:
      ofile.write(" %d "%(hair_color))
  ofile.write("\n")

def write_textures(ofile,tex_set):
  ofile.write(" %d "%len(tex_set)) 
  for tex in tex_set:
    ofile.write(" %s "%tex)
  ofile.write("\n")

def write_voices(ofile, voices):
  ofile.write(" %d "%(len(voices)))
  for voice_rec in voices:
    ofile.write(" %d %s "%(voice_rec[0],voice_rec[1]))
  ofile.write("\n")
    
def export_skins(skins,export_dir):
  ofile = open(export_dir + "skins.txt","w",encoding="utf-8")
  ofile.write("skins_file version 1\n")
  if len(skins) > 16:
    skins = skins[0:15]
  ofile.write("%d\n"%len(skins))
  for skin in skins:
    skin_name      = skin[0]
    skin_flags     = skin[1]
    body_name      = skin[2]
    calf_name      = skin[3]
    hand_name      = skin[4]

    head_mesh      = skin[5]
    face_keys      = skin[6]
    hair_meshes    = skin[7]
    beard_meshes   = skin[8]
    hair_textures  = skin[9]
    beard_textures = skin[10]
    face_textures  = skin[11]
    voices         = skin[12]
    skeleton_name  = skin[13]
    scale          = skin[14]

    blood_particles_1 = 0
    blood_particles_2 = 0
    constraints = []
    if len(skin) > 15:
      blood_particles_1 = skin[15]
    if len(skin) > 16:
      blood_particles_2 = skin[16]
    if len(skin) > 17:
      constraints = skin[17]
    
    ofile.write("%s %d\n %s %s %s\n"%(skin_name, skin_flags, body_name, calf_name, hand_name))
    ofile.write(" %s %d "%(head_mesh,len(face_keys)))
    for face_key in face_keys:
      ofile.write("skinkey_%s %d %d %f %f %s "%(convert_to_identifier(face_key[4]), face_key[0],face_key[1],face_key[2],face_key[3],replace_spaces(face_key[4])))
    ofile.write("\n%d\n"%len(hair_meshes))
    for mesh_name in hair_meshes:
      ofile.write(" %s "%mesh_name)
    ofile.write("\n %d\n"%len(beard_meshes)) 
    for bmn in beard_meshes:
      ofile.write("  %s\n"%bmn)
    ofile.write("\n")
    write_textures(ofile,hair_textures)
    write_textures(ofile,beard_textures)
    write_face_tex(ofile,face_textures)
    write_voices(ofile, voices)
    ofile.write(" %s %f "%(skeleton_name, scale))
    ofile.write("\n%d %d\n"%(blood_particles_1, blood_particles_2))
    ofile.write("%d\n"%(len(constraints)))
    for constraint in constraints:
      ofile.write("\n%f %d %d "%(constraint[0], constraint[1], (len(constraint) - 2)))
      for i_pair in range(len(constraint)):
        if i_pair > 1:
          ofile.write(" %f %d"%(constraint[i_pair][0], constraint[i_pair][1]))
    ofile.write("\n")
  ofile.close()


def processSkins(context):
  print( "Exporting skins...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  export_skins(skins,export_dir)



from smart_module.module_system.header_factions import *

####################################################################################################################
#  Each faction record contains the following fields:
#  1) Faction id: used for referencing factions in other files.
#     The prefix fac_ is automatically added before each faction id.
#  2) Faction name.
#  3) Faction flags. See header_factions.py for a list of available flags
#  4) Faction coherence. Relation between members of this faction.
#  5) Relations. This is a list of relation records.
#     Each relation record is a tuple that contains the following fields:
#    5.1) Faction. Which other faction this relation is referring to
#    5.2) Value: Relation value between the two factions.
#         Values range between -1 and 1.
#  6) Ranks
#  7) Faction color (default is gray)
####################################################################################################################

default_kingdom_relations = [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.05),("mountain_bandits", -0.02),("forest_bandits", -0.02)]
factions = [
  ("no_faction","No Faction",0, 0.9, [], []),
  ("commoners","Commoners",0, 0.1,[("player_faction",0.1)], []),
  ("outlaws","Outlaws", max_player_rating(-30), 0.5,[("commoners",-0.6),("player_faction",-0.15)], [], 0x888888),
# Factions before this point are hardwired into the game end their order should not be changed.

  ("neutral","Neutral",0, 0.1,[("player_faction",0.0)], [],0xFFFFFF),
  ("innocents","Innocents", ff_always_hide_label, 0.5,[("outlaws",-0.05)], []),
  ("merchants","Merchants", ff_always_hide_label, 0.5,[("outlaws",-0.5),], []),

  ("dark_knights","{!}Dark Knights", 0, 0.5,[("innocents",-0.9),("player_faction",-0.4)], []),

  ("culture_1",  "{!}culture_1", 0, 0.9, [], []),
  ("culture_2",  "{!}culture_2", 0, 0.9, [], []),
  ("culture_3",  "{!}culture_3", 0, 0.9, [], []),
  ("culture_4",  "{!}culture_4", 0, 0.9, [], []),
  ("culture_5",  "{!}culture_5", 0, 0.9, [], []),
  ("culture_6",  "{!}culture_6", 0, 0.9, [], []),

#  ("swadian_caravans","Swadian Caravans", 0, 0.5,[("outlaws",-0.8), ("dark_knights",-0.2)], []),
#  ("vaegir_caravans","Vaegir Caravans", 0, 0.5,[("outlaws",-0.8), ("dark_knights",-0.2)], []),

  ("player_faction","Player Faction",0, 0.9, [], []),
  ("player_supporters_faction","Player's Supporters",0, 0.9, [("player_faction",1.00),("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xFF4433), #changed name so that can tell difference if shows up on map
  ("kingdom_1",  "Kingdom of Swadia", 0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xEE7744),
  ("kingdom_2",  "Kingdom of Vaegirs",    0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xCCBB99),
  ("kingdom_3",  "Khergit Khanate", 0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xCC99FF),
  ("kingdom_4",  "Kingdom of Nords",    0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0x33DDDD),
  ("kingdom_5",  "Kingdom of Rhodoks",  0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0x33DD33),
  ("kingdom_6",  "Sarranid Sultanate",  0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xDDDD33),

##  ("kingdom_1_rebels",  "Swadian rebels", 0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xCC2211),
##  ("kingdom_2_rebels",  "Vaegir rebels",    0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xCC2211),
##  ("kingdom_3_rebels",  "Khergit rebels", 0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xCC2211),
##  ("kingdom_4_rebels",  "Nord rebels",    0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xCC2211),
##  ("kingdom_5_rebels",  "Rhodok rebels",  0, 0.9, [("outlaws",-0.05),("peasant_rebels", -0.1),("deserters", -0.02),("mountain_bandits", -0.05),("forest_bandits", -0.05)], [], 0xCC2211),

  ("kingdoms_end","{!}kingdoms_end", 0, 0,[], []),

  ("robber_knights",  "{!}robber_knights", 0, 0.1, [], []),

  ("khergits","{!}Khergits", 0, 0.5,[("player_faction",0.0)], []),
  ("black_khergits","{!}Black Khergits", 0, 0.5,[("player_faction",-0.3),("kingdom_1",-0.02),("kingdom_2",-0.02)], []),

##  ("rebel_peasants","Rebel Peasants", 0, 0.5,[("vaegirs",-0.5),("player_faction",0.0)], []),

  ("manhunters","Manhunters", 0, 0.5,[("outlaws",-0.6),("player_faction",0.1)], []),
  ("deserters","Deserters", 0, 0.5,[("manhunters",-0.6),("merchants",-0.5),("player_faction",-0.1)], [], 0x888888),
  ("mountain_bandits","Mountain Bandits", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15)], [], 0x888888),
  ("forest_bandits","Forest Bandits", 0, 0.5,[("commoners",-0.2),("merchants",-0.5),("manhunters",-0.6),("player_faction",-0.15)], [], 0x888888),

  ("undeads","{!}Undeads", max_player_rating(-30), 0.5,[("commoners",-0.7),("player_faction",-0.5)], []),
  ("slavers","{!}Slavers", 0, 0.1, [], []),
  ("peasant_rebels","{!}Peasant Rebels", 0, 1.0,[("noble_refugees",-1.0),("player_faction",-0.4)], []),
  ("noble_refugees","{!}Noble Refugees", 0, 0.5,[], []),
]
